From 8b91ed2e66c620e73c96a586bf668b2ec4c24462 Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa.rosenzweig@collabora.com>
Date: Thu, 11 Feb 2021 15:23:01 -0500
Subject: [PATCH] pan/bi: Fix jumps to terminal block again

New scheduler broke this.

Fixes: 77933d16d8c ("pan/bi: Switch to new scheduler")
Reported-by: Icecream95 <ixn@disroot.org>
Signed-off-by: Alyssa Rosenzweig <alyssa.rosenzweig@collabora.com>
---
 src/panfrost/bifrost/bifrost_compile.c | 7 ++++++-
 1 file changed, 6 insertions(+), 1 deletion(-)

diff --git a/src/panfrost/bifrost/bifrost_compile.c b/src/panfrost/bifrost/bifrost_compile.c
index 1a90d947c99..cfdf05d02e8 100644
--- a/src/panfrost/bifrost/bifrost_compile.c
+++ b/src/panfrost/bifrost/bifrost_compile.c
@@ -2484,7 +2484,9 @@ bifrost_nir_lower_i8_fragout(nir_shader *shader)
 }
 
 /* Dead code elimination for branches at the end of a block - only one branch
- * per block is legal semantically, but unreachable jumps can be generated */
+ * per block is legal semantically, but unreachable jumps can be generated.
+ * Likewise we can generate jumps to the terminal block which need to be
+ * lowered away to a jump to #0x0, which induces successful termination. */
 
 static void
 bi_cull_dead_branch(bi_block *block)
@@ -2502,6 +2504,9 @@ bi_cull_dead_branch(bi_block *block)
 
                 branched = true;
                 was_jump = ins->op == BI_OPCODE_JUMP;
+
+                if (bi_is_terminal_block(block))
+                        ins->branch_target = NULL;
         }
 }
 
-- 
2.27.0

